from django.dispatch import Signal


invitation_added = Signal(providing_args=['user', 'count'])

invitation_sent = Signal()

invitation_accepted = Signal(providing_args=['inviting_user', 'new_user'])



from django.core.urlresolvers import reverse
from django.core import mail
from django.contrib.auth.models import User
from utils import BaseTestCase
from invitation.models import Invitation


class InviteOnlyModeTestCase(BaseTestCase):
    urls = 'invitation.tests.invite_only_urls'

    def test_invation_mode(self):
        # Normal registration view should redirect
        response = self.client.get(reverse('registration_register'))
        self.assertRedirects(response, reverse('invitation_invite_only'))
        # But registration after invitation view should work
        response = self.client.get(reverse('invitation_register',
                                           args=('A' * 40,)))
        self.assertRedirects(response, reverse('registration_disallowed'))

    def test_invitation(self):
        self.client.login(username='testuser', password='testuser')
        response = self.client.post(reverse('invitation_invite'),
                                    {'email': 'friend@example.com'})
        self.assertRedirects(response, reverse('invitation_complete'))
        self.assertEqual(len(mail.outbox), 1)

    def test_registration(self):
        # Make sure error message is shown in
        # case of an invalid invitation key
        response = self.client.get(reverse('invitation_register',
                                           args=('A' * 40,)))
        self.assertRedirects(response, reverse('registration_disallowed'))
        # Registration with an invitation
        invitation = Invitation.objects.invite(self.user(),
                                               'friend@example.com')
        register_url = reverse('invitation_register', args=(invitation.key,))
        response = self.client.get(register_url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response,
                                'registration/registration_form.html')
        self.assertContains(response, invitation.email)
        # We are posting a different email than the invitation.email and the
        # form should not ignore it and register that new email
        response = self.client.post(register_url,
                                    {'username': u'friend',
                                     'email': u'other@example.com',
                                     'password1': u'friend',
                                     'password2': u'friend'})
        self.assertRedirects(response, '/users/friend/')
        self.assertEqual(len(mail.outbox), 0)       # No confirmation email
        new_user = User.objects.get(username='friend')
        self.assertEqual(new_user.email, 'other@example.com')
        self.assertEqual(new_user.is_active, True)
        self.assertRaises(Invitation.DoesNotExist,
                          Invitation.objects.get,
                          user=self.user(),
                          email='friend@example.com')


class InviteOptionalModeTestCase(BaseTestCase):
    urls = 'invitation.tests.invite_optional_urls'

    def test_invation_mode(self):
        # Normal registration view should work
        response = self.client.get(reverse('registration_register'))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response,
                                'registration/registration_register.html')
        # So as registration after invitation view
        response = self.client.get(reverse('invitation_register',
                                           args=('A' * 40,)))
        self.assertRedirects(response, reverse('registration_disallowed'))

    def test_invitation(self):
        self.client.login(username='testuser', password='testuser')
        response = self.client.get(reverse('invitation_invite'))
        self.assertEqual(response.status_code, 200)
        response = self.client.post(reverse('invitation_invite'),
                                    {'email': 'friend@example.com'})
        self.assertRedirects(response, reverse('invitation_complete'))
        invitation_query = Invitation.objects.filter(user=self.user(),
                                                   email='friend@example.com')
        self.assertEqual(invitation_query.count(), 1)
        self.assertEqual(len(mail.outbox), 1)



import datetime
from django.core import mail
from django.contrib.auth.models import User
from django.utils.timezone import now

from utils import BaseTestCase
from invitation import app_settings
from invitation.models import Invitation


EXPIRE_DAYS = app_settings.EXPIRE_DAYS


class InvitationTestCase(BaseTestCase):
    def setUp(self):
        super(InvitationTestCase, self).setUp()
        user = self.user()
        self.invitation = Invitation.objects.create(user=user,
                                                    email=u'test@example.com',
                                                    key=u'F' * 40)

    def make_invalid(self, invitation=None):
        invitation = invitation or self.invitation
        invitation.date_invited = now() - datetime.timedelta(EXPIRE_DAYS + 10)
        invitation.save()
        return invitation

    def test_send_email(self):
        self.invitation.send_email()
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].recipients()[0], u'test@example.com')
        self.invitation.send_email(u'other@email.org')
        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[1].recipients()[0], u'other@email.org')

    def test_mark_accepted(self):
        new_user = User.objects.create_user('test', 'test@example.com', 'test')
        pk = self.invitation.pk
        self.invitation.mark_accepted(new_user)
        self.assertRaises(Invitation.DoesNotExist,
                          Invitation.objects.get, pk=pk)

    def test_invite(self):
        Invitation.objects.all().delete()
        invitation = Invitation.objects.invite(self.user(), 'test@example.com')
        self.assertEqual(invitation.user, self.user())
        self.assertEqual(invitation.email, 'test@example.com')
        self.assertEqual(len(invitation.key), 40)
        self.assertEqual(invitation.is_valid(), True)
        self.assertEqual(type(invitation.expiration_date()), datetime.date)
        # Test if existing valid record is returned
        # when we try with the same credentials
        self.assertEqual(Invitation.objects.invite(self.user(),
                                              'test@example.com'), invitation)
        # Try with an invalid invitation
        invitation = self.make_invalid(invitation)
        new_invitation = Invitation.objects.invite(self.user(),
                                                   'test@example.com')
        self.assertEqual(new_invitation.is_valid(), True)
        self.assertNotEqual(new_invitation, invitation)

    def test_find(self):
        self.assertEqual(Invitation.objects.find(self.invitation.key),
                         self.invitation)
        invitation = self.make_invalid()
        self.assertEqual(invitation.is_valid(), False)
        self.assertRaises(Invitation.DoesNotExist,
                          Invitation.objects.find, invitation.key)
        self.assertEqual(Invitation.objects.all().count(), 0)
        self.assertRaises(Invitation.DoesNotExist,
                          Invitation.objects.find, '')



from __future__ import absolute_import

from django.contrib.auth import authenticate, login
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.core.exceptions import ImproperlyConfigured
from django.shortcuts import redirect
from django.utils.decorators import method_decorator
from django.utils.encoding import force_text
from django.views.generic.edit import FormView

from registration.signals import user_registered
from registration.views import RegistrationView as BaseRegistrationView

from .models import InvitationError, Invitation
from .forms import InvitationForm


class RegistrationView(BaseRegistrationView):
    """Registration via invitation key."""

    def get_initial(self):
        """Sets the invitation's email as the initial value for the form."""
        return {'email': self.invitation.email}

    def get_success_url(self, request, user):
        return (user.get_absolute_url(), (), {})

    def registration_allowed(self, request):
        """Search for a valid invitation key."""
        invitation_key = self.kwargs.get('invitation_key')
        try:
            self.invitation = Invitation.objects.find(invitation_key)
        except Invitation.DoesNotExist:
            return False
        return True

    def register(self, request, **cleaned_data):
        """Allow a new user to register via invitation."""
        username, email, password = cleaned_data['username'], \
                                    cleaned_data['email'], \
                                    cleaned_data['password1']
        User.objects.create_user(username, email, password)
        self.user = authenticate(username=username, password=password)
        login(request, self.user)
        user_registered.send(sender=self.__class__, user=self.user,
                             request=request)
        self.invitation.mark_accepted(self.user)
        return self.user


class InvitationView(FormView):
    """Create an invitation and send invitation email."""
    form_class = InvitationForm
    http_method_names = ['get', 'post', 'head', 'options', 'trace']
    invitation_error_url = 'invitation_unavailable'
    success_url = 'invitation_complete'
    template_name = 'invitation/invitation_form.html'

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(InvitationView, self).dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        try:
            self.invitation = Invitation.objects.invite(
                self.request.user, form.cleaned_data['email']
            )
        except InvitationError:
            return redirect(self.get_invitation_error_url())
        self.invitation.send_email()
        return redirect(self.get_success_url())

    def get_invitation_error_url(self):
        """Returns the supplied invitation_error_url."""
        if self.invitation_error_url:
            # Forcing possible reverse_lazy evaluation
            url = force_text(self.invitation_error_url)
        else:
            raise ImproperlyConfigured(
                'No url to redirect to. Provide a invitation_error_url'
            )
        return url



from __future__ import absolute_import

from django.conf.urls import patterns, url
from django.views.generic.base import RedirectView, TemplateView
from django.contrib.auth.decorators import login_required

from .app_settings import INVITE_ONLY
from .views import InvitationView, RegistrationView


login_required_direct_to_template = login_required(TemplateView.as_view())


urlpatterns = patterns('',
    url(r'^invite/$', InvitationView.as_view(),
        name='invitation_invite'),
    url(r'^invite/complete/$',
        TemplateView.as_view(template_name='invitation/invitation_complete.html'),
        name='invitation_complete'),
    url(r'^invite/unavailable/$',
        TemplateView.as_view(template_name='invitation/invitation_unavailable.html'),
        name='invitation_unavailable'),
    url(r'^accept/complete/$',
        TemplateView.as_view(template_name='invitation/invitation_registered.html'),
        name='invitation_registered'),
    url(r'^accept/(?P<invitation_key>\w+)/$',
        RegistrationView.as_view(), name='invitation_register'),
    url(r'^wrong-key/$',
        TemplateView.as_view(template_name='invitation/wrong_invitation_key.html'),
        name='registration_disallowed')
)


if INVITE_ONLY:
    urlpatterns += patterns('',
        url(r'^register/$',
            RedirectView.as_view(url='../invite_only/', permanent=False),
            name='registration_register'),
        url(r'^invite_only/$',
            TemplateView.as_view(template_name='invitation/invite_only.html'),
            name='invitation_invite_only'),
    )



from __future__ import absolute_import

from django.contrib import admin

from .models import Invitation


class InvitationAdmin(admin.ModelAdmin):
    list_display = ('user', 'email', 'expiration_date')
admin.site.register(Invitation, InvitationAdmin)



import datetime
import hashlib
import random

from django.core.mail import send_mail
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sites.models import Site, RequestSite
from django.db import models
from django.template.loader import render_to_string
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

import app_settings
import signals


class InvitationError(Exception):
    pass


class InvitationManager(models.Manager):
    def invite(self, user, email):
        """Get or create an invitation for ``email`` from ``user``.

        This method doesn't an send email. You need to call ``send_email()``
        method on returned ``Invitation`` instance.
        """
        invitation = None
        try:
            # It is possible that there is more than one invitation fitting
            # the criteria. Normally this means some older invitations are
            # expired or an email is invited consequtively.
            invitation = self.filter(user=user, email=email)[0]
            if not invitation.is_valid():
                invitation = None
        except (Invitation.DoesNotExist, IndexError):
            pass
        if invitation is None:
            key = '%s%0.16f%s%s' % (settings.SECRET_KEY, random.random(),
                                    user.email, email)
            key = hashlib.sha1(key).hexdigest()
            invitation = self.create(user=user, email=email, key=key)
        return invitation
    invite.alters_data = True

    def find(self, invitation_key):
        """Find a valid invitation for the given key or raise
        ``Invitation.DoesNotExist``.

        This function always returns a valid invitation. If an invitation is
        found but not valid it will be automatically deleted.
        """
        try:
            invitation = self.filter(key=invitation_key)[0]
        except IndexError:
            raise Invitation.DoesNotExist
        if not invitation.is_valid():
            invitation.delete()
            raise Invitation.DoesNotExist
        return invitation

    def valid(self):
        """Filter valid invitations."""
        expiration = now() - datetime.timedelta(app_settings.EXPIRE_DAYS)
        return self.get_query_set().filter(date_invited__gte=expiration)

    def invalid(self):
        """Filter invalid invitation."""
        expiration = now() - datetime.timedelta(app_settings.EXPIRE_DAYS)
        return self.get_query_set().filter(date_invited__le=expiration)

    def delete_expired_keys(self):
        """Removes expired instances of ``Invitation``.

        Invitation keys to be deleted are identified by searching for
        instances of ``Invitation`` with difference between now and
        `date_invited` date greater than ``EXPIRE_DAYS``.

        It is recommended that this method be executed regularly as
        part of your routine site maintenance; this application
        provides a custom management command which will call this
        method, accessible as ``manage.py cleanupinvitations``.
        """
        return self.invalid().delete()


class Invitation(models.Model):
    user = models.ForeignKey(User, related_name='invitations')
    email = models.EmailField(_(u'e-mail'))
    key = models.CharField(_(u'invitation key'), max_length=40, unique=True)
    date_invited = models.DateTimeField(_(u'date invited'), default=now())

    objects = InvitationManager()

    class Meta:
        verbose_name = _(u'invitation')
        verbose_name_plural = _(u'invitations')
        ordering = ('-date_invited',)

    def __unicode__(self):
        return _('%(username)s invited %(email)s on %(date)s') % {
            'username': self.user.username,
            'email': self.email,
            'date': str(self.date_invited.date()),
        }

    @models.permalink
    def get_absolute_url(self):
        return ('invitation_register', (), {'invitation_key': self.key})

    @property
    def _expires_at(self):
        return self.date_invited + datetime.timedelta(app_settings.EXPIRE_DAYS)

    def is_valid(self):
        """Return ``True`` if the invitation is still valid, ``False``
        otherwise.
        """
        return now() < self._expires_at

    def expiration_date(self):
        """Return a ``datetime.date()`` object representing expiration date.
        """
        return self._expires_at.date()
    expiration_date.short_description = _(u'expiration date')
    expiration_date.admin_order_field = 'date_invited'

    def send_email(self, email=None, site=None, request=None):
        """Send invitation email.

        Both ``email`` and ``site`` parameters are optional. If not supplied
        instance's ``email`` field and current site will be used.

        **Templates:**

        :invitation/invitation_email_subject.txt:
            Template used to render the email subject.

            **Context:**

            :invitation: ``Invitation`` instance ``send_email`` is called on.
            :site: ``Site`` instance to be used.

        :invitation/invitation_email.txt:
            Template used to render the email body.

            **Context:**

            :invitation: ``Invitation`` instance ``send_email`` is called on.
            :expiration_days: ``INVITATION_EXPIRE_DAYS`` setting.
            :site: ``Site`` instance to be used.

        **Signals:**

        ``invitation.signals.invitation_sent`` is sent on completion.
        """
        email = email or self.email
        if site is None:
            if Site._meta.installed:
                site = Site.objects.get_current()
            elif request is not None:
                site = RequestSite(request)
        subject = render_to_string('invitation/invitation_email_subject.txt',
                                   {'invitation': self, 'site': site})
        # Email subject *must not* contain newlines
        subject = ''.join(subject.splitlines())
        message = render_to_string('invitation/invitation_email.txt', {
            'invitation': self,
            'expiration_days': app_settings.EXPIRE_DAYS,
            'site': site
        })
        send_mail(subject, message, settings.DEFAULT_FROM_EMAIL, [email])
        signals.invitation_sent.send(sender=self)

    def mark_accepted(self, new_user):
        """Update sender's invitation statistics and delete self.

        ``invitation.signals.invitation_accepted`` is sent just before the
        instance is deleted.
        """
        signals.invitation_accepted.send(sender=self,
                                         inviting_user=self.user,
                                         new_user=new_user)
        self.delete()
    mark_accepted.alters_data = True



VERSION = (0, 1, 0, 'beta', 3)

__author__ = u'Daniel Barreto'
__credits__ = [u'Atamert \xd6l\xe7gen']


__license__ = 'BSD'
__maintainer__ = u'Daniel Barreto'
__email__ = 'daniel.barreto.n@gmail.com'
__status__ = 'Beta'

def get_version():
    from django.utils.version import get_version as django_get_version
    return django_get_version(VERSION) # pragma: no cover



import os
from django.conf import settings
from django.test import TestCase
from django.contrib.auth.models import User


class BaseTestCase(TestCase):
    def setUp(self):
        super(BaseTestCase, self).setUp()
        settings.TEMPLATE_DIRS = (
            os.path.join(os.path.dirname(__file__), 'templates'),
        )
        User.objects.create_user('testuser',
                                 'testuser@example.com',
                                 'testuser')

    def user(self):
        return User.objects.get(username='testuser')



from django.utils.importlib import import_module
from invitation import app_settings


app_settings.INVITE_ONLY = True
reload(import_module('invitation.urls'))
reload(import_module('invitation.tests.urls'))
from invitation.tests.urls import urlpatterns



import os
import sys
from distutils.core import setup
from invitation import get_version, __maintainer__, __email__


def compile_translations():
    try:
        from django.core.management.commands.compilemessages \
                                                       import compile_messages
    except ImportError:
        return None
    curdir = os.getcwdu()
    os.chdir(os.path.join(os.path.dirname(__file__), 'invitation'))
    try:
        compile_messages(stderr=sys.stderr)
    except TypeError:
        # compile_messages doesn't accept stderr parameter prior to 1.2.4
        compile_messages()
    os.chdir(curdir)
compile_translations()


license_text = open('LICENSE.txt').read()
long_description = open('README.rst').read()


setup(
    name = 'django-invitation-backend',
    version = get_version().replace(' ', '-'),
    url = 'http://github.com/volrath/django-invitation-backend',
    author = __maintainer__,
    author_email = __email__,
    license = license_text,
    packages = ['invitation',
                'invitation.tests'],
    package_data= {
        'invitation': ['tests/templates/invitation/*',
                       'tests/templates/registration/*',
                       'locale/*/LC_MESSAGES/django.*']
    },
    data_files=[('', ['LICENSE.txt',
                      'README.rst'])],
    description = 'Registration through invitations backend for django-registration > 0.9',
    long_description=long_description,
    classifiers = ['Development Status :: 4 - Beta',
                   'Environment :: Web Environment',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Topic :: Internet :: WWW/HTTP :: Dynamic Content']
)



from django.conf.urls import patterns, url
from django.views.generic.base import TemplateView

import invitation.urls as invitation_urls


urlpatterns = invitation_urls.urlpatterns + patterns(
    '',
    url(r'^register/$',
        TemplateView.as_view(template_name='registration/registration_register.html'),
        name='registration_register'),
    url(r'^register/complete/$',
        TemplateView.as_view(template_name='registration/registration_complete.html'),
        name='registration_complete'),
    url(r'^users/(?P<username>[\w.@+-]+)/',
        TemplateView.as_view(template_name='invitation/user_profile.html'),
        name='user_profile')
)
